/* ----------------------------------------------------------------------------
*
* Raima Database Manager
*
* Copyright (c) 2013 Raima Inc.,  All rights reserved.
*
* Use of this software, whether in source code format, or in executable,
* binary object code form, is governed by the Raima LICENSE which
* is fully described in the LICENSE.TXT file, included within this
* distribution of files.
*
* ----------------------------------------------------------------------------
*/

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* CursorTime_db.h was generated from the DDL file CursorTime_db.ddl. 
* See subsequent examples for DDL modification.
*/
#include "CursorTime_db.h"

/* EXAMPLE - Time Datatype using Cursors */
/* Each time you run this example, two new records are created in the
* database, then all existing records are read from the database
* and printed.
*
* The DDL (Database Definition Language) is in CursorTime_db.ddl. The database
* has been pre-built and initialized.
*
* The following RDM Functions are illustrated:
*      d_opentask/d_closetask
*      d_open/d_iclose
*      d_cursorAlloc/dc_cursorFree
*      d_trbegin/d_trend
*      d_createRecord
*      d_fillnew
*      dc_getFields
*      dc_first/dc_next
*      d_recread
*      dt_timeNow/dt_timeFromString/dt_timeToString
* They are fully documented in http://www.raima.com/support/online-documentation/.
*
* Each of these functions returns an integer status code, where the
* value S_OKAY indicates a successful call.
*
* For simplicity, this example does not check all return codes, but good
* programming practices would dictate that they are checked after each
* RDM call.
*
* The actual database is stored in a directory named 'CursorTime_db' in this
* project directory.
*/

int32_t main (void) 
{
    RDM_TASK task;
    int32_t iStatus;

    /* Create the context for the database operations */
    iStatus = d_opentask (&task);
    if (iStatus == S_OKAY)
    {
        /* Open the database for exclusive access */
        iStatus = d_open ("CursorTime_db", "o", task);

        if (iStatus == S_OKAY)
        {
            struct TimeRecord Time_input[2];
            RDM_CURSOR cursor = NULL;
            TIME_VAL rdmTime;

            iStatus = d_cursorAlloc (&cursor, task, 0);

            /* Create a record in the new database */
			dt_timeNow (&Time_input[0].timeCol);
            rdmTime = dt_timeFromString ("6:02:25.5");
			Time_input[1].timeCol = rdmTime;

            if (iStatus == S_OKAY)
            {
                /* Start a Insert database transaction. */
                iStatus = d_trbegin ("Insert", task);
                if (iStatus == S_OKAY)
                {
                    uint32_t counter;

                    iStatus = d_reclock (TIMERECORD, "w", task, 0);

                    for (counter = 0; iStatus == S_OKAY && counter < sizeof (Time_input) / sizeof (struct TimeRecord); counter++)
                    {
						/* Create a record in the new database */
                        iStatus = d_createRecord (TIMERECORD, &Time_input[counter],
                                sizeof(*Time_input), cursor, task,
                                dc_getDBN (cursor));
                    }

                    iStatus = d_trend (task);
                } 

                /* Start Retrieving data from the Database. */
                if (iStatus == S_OKAY)
                {
                    /* Scan and read all created records.
                    * Two additional records will be created each time you run this example.
                    */
                    iStatus = d_getRecords (TIMERECORD, cursor, task,
                        dc_getDBN (cursor));
                }
                if (iStatus == S_OKAY)
                    iStatus = dc_first (cursor);

                while (iStatus == S_OKAY)
                {
                    struct TimeRecord recS;
                    iStatus = dc_getFields (cursor, TIMERECORD, &recS,
                        sizeof(recS), NULL);
                    if (iStatus == S_OKAY)
                    {
                        char outBuffer[100];
						dt_timeToString (recS.timeCol, outBuffer, sizeof (outBuffer));
                        printf ("Time Retrieved: %s\n", outBuffer);
                        iStatus = dc_next (cursor);
                    }
                }

                iStatus = dc_cursorFree (cursor);
            }
            /* Close the database */
            if (iStatus == S_OKAY)
                iStatus = d_iclose (task, CURR_DB);
            else
                d_iclose (task, CURR_DB);
        }

        if (iStatus == S_OKAY)
            iStatus = d_closetask (task);
        else
            d_closetask (task);
    }

    printf ("Press <Enter> to continue . . .\n");
    getchar ();
}
